@@ -113,19 +113,15 @@ class TransmitterBase(unittest.TestCase):
 		self.assertEqual(
 			self.transm.proceed(["get", jail, cmd]), (0, []))
 		for n, value in enumerate(values):
-			self.assertEqual(
-				self.transm.proceed(["set", jail, cmdAdd, value]),
-				(0, values[:n+1]))
-			self.assertEqual(
-				self.transm.proceed(["get", jail, cmd]),
-				(0, values[:n+1]))
+			ret = self.transm.proceed(["set", jail, cmdAdd, value])
+			self.assertEqual((ret[0], sorted(ret[1])), (0, sorted(values[:n+1])))
+			ret = self.transm.proceed(["get", jail, cmd])
+			self.assertEqual((ret[0], sorted(ret[1])), (0, sorted(values[:n+1])))
 		for n, value in enumerate(values):
-			self.assertEqual(
-				self.transm.proceed(["set", jail, cmdDel, value]),
-				(0, values[n+1:]))
-			self.assertEqual(
-				self.transm.proceed(["get", jail, cmd]),
-				(0, values[n+1:]))
+			ret = self.transm.proceed(["set", jail, cmdDel, value])
+			self.assertEqual((ret[0], sorted(ret[1])), (0, sorted(values[n+1:])))
+			ret = self.transm.proceed(["get", jail, cmd])
+			self.assertEqual((ret[0], sorted(ret[1])), (0, sorted(values[n+1:])))
 
 	def jailAddDelRegexTest(self, cmd, inValues, outValues, jail):
 		cmdAdd = "add" + cmd
@@ -168,8 +164,9 @@ class Transmitter(TransmitterBase):
 		t0 = time.time()
 		self.assertEqual(self.transm.proceed(["sleep", "1"]), (0, None))
 		t1 = time.time()
-		# Approx 1 second delay
-		self.assertAlmostEqual(t1 - t0, 1, places=1)
+		# Approx 1 second delay but not faster
+		dt = t1 - t0
+		self.assertTrue(0.99 < dt < 1.1, msg="Sleep was %g sec" % dt)
 
 	def testDatabase(self):
 		tmp, tmpFilename = tempfile.mkstemp(".db", "fail2ban_")
@@ -231,7 +228,7 @@ class Transmitter(TransmitterBase):
 		time.sleep(1)
 		self.assertEqual(
 			self.transm.proceed(["stop", self.jailName]), (0, None))
-		self.assertTrue(self.jailName not in self.server._Server__jails)
+		self.assertNotIn(self.jailName, self.server._Server__jails)
 
 	def testStartStopAllJail(self):
 		self.server.addJail("TestJail2", "auto")
@@ -245,8 +242,8 @@ class Transmitter(TransmitterBase):
 		time.sleep(0.1)
 		self.assertEqual(self.transm.proceed(["stop", "all"]), (0, None))
 		time.sleep(1)
-		self.assertTrue(self.jailName not in self.server._Server__jails)
-		self.assertTrue("TestJail2" not in self.server._Server__jails)
+		self.assertNotIn(self.jailName, self.server._Server__jails)
+		self.assertNotIn("TestJail2", self.server._Server__jails)
 
 	def testJailIdle(self):
 		self.assertEqual(
@@ -691,10 +688,7 @@ class Transmitter(TransmitterBase):
 
 	def testJournalMatch(self):
 		if not filtersystemd: # pragma: no cover
-			if sys.version_info >= (2, 7):
-				raise unittest.SkipTest(
-					"systemd python interface not available")
-			return
+			raise unittest.SkipTest("systemd python interface not available")
 		jailName = "TestJail2"
 		self.server.addJail(jailName, "systemd")
 		values = [
@@ -794,10 +788,8 @@ class TransmitterLogging(TransmitterBase):
 		self.setGetTest("logtarget", "STDERR")
 
 	def testLogTargetSYSLOG(self):
-		if not os.path.exists("/dev/log") and sys.version_info >= (2, 7):
+		if not os.path.exists("/dev/log"):
 			raise unittest.SkipTest("'/dev/log' not present")
-		elif not os.path.exists("/dev/log"):
-			return
 		self.assertTrue(self.server.getSyslogSocket(), "auto")
 		self.setGetTest("logtarget", "SYSLOG")
 		self.assertTrue(self.server.getSyslogSocket(), "/dev/log")
@@ -934,7 +926,7 @@ class LoggingTests(LogCaptureTestCase):
 			badThread = _BadThread()
 			badThread.start()
 			badThread.join()
-			self.assertTrue(self._is_logged("Unhandled exception"))
+			self.assertLogged("Unhandled exception")
 		finally:
 			sys.__excepthook__ = prev_exchook
 		self.assertEqual(len(x), 1)
